/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.gui;

import jalview.datamodel.AlignmentAnnotation;
import jalview.datamodel.AlignmentColsCollection;
import jalview.datamodel.AlignmentRowsCollection;
import jalview.datamodel.Annotation;
import jalview.datamodel.SequenceI;
import jalview.renderer.seqfeatures.FeatureColourFinder;
import jalview.viewmodel.OverviewDimensions;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.image.BufferedImage;

public class OverviewRenderer
{
  private FeatureColourFinder finder;

  private SequenceRenderer sr;

  // image to render on
  private BufferedImage miniMe;

  // raw number of pixels to allocate to each column
  private float pixelsPerCol;

  // raw number of pixels to allocate to each row
  private float pixelsPerSeq;

  public OverviewRenderer(SequenceRenderer seqRenderer,
          FeatureColourFinder colfinder, OverviewDimensions od)
  {
    sr = seqRenderer;
    finder = colfinder;

    pixelsPerCol = od.getPixelsPerCol();
    pixelsPerSeq = od.getPixelsPerSeq();
    miniMe = new BufferedImage(od.getWidth(), od.getHeight(),
            BufferedImage.TYPE_INT_RGB);
  }

  /**
   * Draw alignment rows and columns onto an image
   * 
   * @param rit
   *          Iterator over rows to be drawn
   * @param cit
   *          Iterator over columns to be drawn
   * @return image containing the drawing
   */
  public BufferedImage draw(AlignmentRowsCollection rows,
          AlignmentColsCollection cols)
  {
    int rgbcolor = Color.white.getRGB();
    int seqIndex = 0;
    int pixelRow = 0;
    for (int alignmentRow : rows)
    {
      // get details of this alignment row
      boolean hidden = rows.isHidden(alignmentRow);
      SequenceI seq = rows.getSequence(alignmentRow);

      // calculate where this row extends to in pixels
      int endRow = Math.min(Math.round((seqIndex + 1) * pixelsPerSeq) - 1,
              miniMe.getHeight() - 1);

      int colIndex = 0;
      int pixelCol = 0;
      for (int alignmentCol : cols)
      {
        // calculate where this column extends to in pixels
        int endCol = Math.min(
                Math.round((colIndex + 1) * pixelsPerCol) - 1,
                miniMe.getWidth() - 1);

        // determine the colour based on the sequence and column position
        rgbcolor = getColumnColourFromSequence(seq,
                hidden || cols.isHidden(alignmentCol), alignmentCol, finder);

        // fill in the appropriate number of pixels
        for (int row = pixelRow; row <= endRow; ++row)
        {
          for (int col = pixelCol; col <= endCol; ++col)
          {
            miniMe.setRGB(col, row, rgbcolor);
          }
        }

        pixelCol = endCol + 1;
        colIndex++;
      }
      pixelRow = endRow + 1;
      seqIndex++;
    }
    return miniMe;
  }

  /*
   * Find the colour of a sequence at a specified column position
   */
  private int getColumnColourFromSequence(jalview.datamodel.SequenceI seq,
          boolean isHidden, int lastcol, FeatureColourFinder finder)
  {
    Color color = Color.white;

    if ((seq != null) && (seq.getLength() > lastcol))
    {
      color = sr.getResidueColour(seq, lastcol, finder);
    }

    if (isHidden)
    {
      color = color.darker().darker();
    }

    return color.getRGB();
  }

  public void drawGraph(Graphics g, AlignmentAnnotation _aa, int charWidth,
          int y, AlignmentColsCollection cols)
  {
    Annotation[] aa_annotations = _aa.annotations;
    g.setColor(Color.white);
    g.fillRect(0, 0, miniMe.getWidth(), y);
    // g.setColor(new Color(0, 0, 180));

    int height;

    int colIndex = 0;
    int pixelCol = 0;
    for (int alignmentCol : cols)
    {
      int endCol = Math.min(Math.round((colIndex + 1) * pixelsPerCol) - 1,
              miniMe.getWidth() - 1);

      if (alignmentCol < aa_annotations.length
              && aa_annotations[alignmentCol] != null)
      {
        if (aa_annotations[alignmentCol].colour == null)
        {
          g.setColor(Color.black);
        }
        else
        {
          g.setColor(aa_annotations[alignmentCol].colour);
        }

        height = (int) ((aa_annotations[alignmentCol].value / _aa.graphMax) * y);
        if (height > y)
        {
          height = y;
        }

        g.fillRect(pixelCol, y - height, endCol - pixelCol + 1, height);
      }
      pixelCol = endCol + 1;
      colIndex++;
    }
  }
}
